@@ -191,6 +191,7 @@ class Firefox(Cleaner):
         self.add_option('download_history', _('Download history'), _('List of files downloaded'))
         self.add_option('forms', _('Form history'), _('A history of forms entered in web sites and in the Search bar'))
         self.add_option('session_restore', _('Session restore'), _('Loads the initial session after the browser closes or crashes'))
+        self.add_option('site_preferences', _('Site preferences'), _('Settings for individual sites'))
         self.add_option('passwords', _('Passwords'), _('A database of usernames and passwords as well as a list of sites that should not store passwords'))
         self.add_option('url_history', _('URL history'), _('List of visited web pages'))
         self.add_option('vacuum', _('Vacuum'), _('Clean database fragmentation to reduce space and improve speed without removing any data'))
@@ -254,6 +255,11 @@ class Firefox(Cleaner):
         # session restore
         if 'session_restore' == option_id:
             files += FileUtilities.expand_glob_join(self.profile_dir, "sessionstore.js")
+            files += FileUtilities.expand_glob_join(self.profile_dir, "sessionstore.bak")
+        # site-specific preferences
+        if 'site_preferences' == option_id:
+            files += FileUtilities.expand_glob_join(self.profile_dir, "content-prefs.sqlite")
+
         # URL history
         if 'url_history' == option_id:
             # Firefox version 1
@@ -382,7 +388,6 @@ class System(Cleaner):
             # TRANSLATORS: 'Update' is a noun, and 'Update uninstallers' is an option to delete
             # the uninstallers for software updates.
             self.add_option('updates', _('Update uninstallers'), _('Delete uninstallers for Microsoft updates including hotfixes, service packs, and Internet Explorer updates'))
-            self.set_warning('updates', _('This option is experimental and may cause system problems.'))
         if HAVE_GTK:
             self.add_option('clipboard', _('Clipboard'), _('The desktop environment\'s clipboard used for copy and paste operations'))
         # TRANSLATORS: 'free' means 'unallocated'
@@ -626,15 +631,16 @@ class System(Cleaner):
     def whitelisted(self, pathname):
         """Return boolean whether file is whitelisted"""
         regexes = [
+            '^/tmp/.X0-lock$',
             '^/tmp/.truecrypt_aux_mnt.*/(control|volume)$',
             '^/tmp/.vbox-[^/]+-ipc/lock$',
             '^/tmp/.wine-[0-9]+/server-.*/lock$',
-            '^/tmp/.X0-lock$',
             '^/tmp/gconfd-[^/]+/lock/ior$',
             '^/tmp/ksocket-[^/]+/(Arts_SoundServerV2|secret-cookie)$',
             '^/tmp/orbit-[^/]+/bonobo-activation-register[a-z0-9-]*.lock$',
             '^/tmp/orbit-[^/]+/bonobo-activation-server-[a-z0-9-]*ior$',
-            '^/tmp/pulse-[^/]+/pid$' ]
+            '^/tmp/pulse-[^/]+/pid$',
+            '^/var/tmp/kdecache-' ]
         for regex in regexes:
             if None != re.match(regex, pathname):
                 return True
@@ -659,6 +665,8 @@ def create_simple_cleaner(paths):
         action_key = '__customfileaction'
         def get_commands(self):
             for path in paths:
+                if not isinstance(path, (str, unicode)):
+                    raise RuntimeError('expected path as string but got %s' % str(path))
                 if os.path.isdir(path):
                     for child in children_in_directory(path, True):
                         yield Command.Shred(child)
