@@ -1,7 +1,7 @@
 # vim: ts=4:sw=4:expandtab
 
 ## BleachBit
-## Copyright (C) 2010 Andrew Ziem
+## Copyright (C) 2011 Andrew Ziem
 ## http://bleachbit.sourceforge.net
 ##
 ## This program is free software: you can redistribute it and/or modify
@@ -156,6 +156,8 @@ class CleanerML:
         """<description> element under <option>"""
         self.option_description = _(getText(description.childNodes))
         self.xlate_cb(self.option_description)
+        translators = description.getAttribute('translators')
+        self.xlate_cb(self.option_description, translators)
 
     def handle_cleaner_option_warning(self, warning):
         """<warning> element under <option>"""
