@@ -2,7 +2,7 @@
 # -*- coding: UTF-8 -*-
 
 ## BleachBit
-## Copyright (C) 2010 Andrew Ziem
+## Copyright (C) 2011 Andrew Ziem
 ## http://bleachbit.sourceforge.net
 ##
 ## This program is free software: you can redistribute it and/or modify
@@ -41,6 +41,11 @@ if 'nt' == os.name:
 def diagnostic_info():
     """Return diagnostic information as a string"""
     s = "BleachBit version %s" % Common.APP_VERSION
+    try:
+        import gtk
+        s += '\nGTK version %s' % '.'.join([str(x) for x in gtk.gtk_version])
+    except:
+        pass
     s += "\nlocal_cleaners_dir = %s" % Common.local_cleaners_dir
     s += "\nlocale_dir = %s" % Common.locale_dir
     s += "\noptions_dir = %s" % Common.options_dir
@@ -48,10 +53,12 @@ def diagnostic_info():
     s += "\nsystem_cleaners_dir = %s" % Common.system_cleaners_dir
     s += "\nlocale.getdefaultlocale = %s" % str(locale.getdefaultlocale())
     if 'posix' == os.name:
-        for value in ('DESKTOP_SESSION', 'LOGNAME', 'USER', 'SUDO_UID'):
-            s += "\nos.getenv('%s') = %s" % (value, os.getenv(value))
+        envs = ('DESKTOP_SESSION', 'LOGNAME', 'USER', 'SUDO_UID')
+    if 'nt' == os.name:
+        envs = ('APPDATA', 'localappdata', 'USERPROFILE')
+    for env in envs:
+        s += "\nos.getenv('%s') = %s" % (env, os.getenv(env))
     s += "\nos.expanduser('~') = %s" % os.path.expanduser('~')
-    s += "\nos.expandvars('$USERPROFILE') = %s" % os.path.expandvars('$USERPROFILE')
     if 'linux2' == sys.platform:
         if hasattr(platform, 'linux_distribution'):
             s += "\nplatform.linux_distribution() = %s" % str(platform.linux_distribution())
