@@ -2,7 +2,7 @@
 # -*- coding: UTF-8 -*-
 
 ## BleachBit
-## Copyright (C) 2010 Andrew Ziem
+## Copyright (C) 2011 Andrew Ziem
 ## http://bleachbit.sourceforge.net
 ##
 ## This program is free software: you can redistribute it and/or modify
@@ -25,25 +25,21 @@ File-related utilities
 """
 
 
-
-import datetime
+import codecs
 import glob
 import locale
 import os
+import random
 import re
 import stat
-import subprocess
+import string
 import sys
 import tempfile
-
-if not "iglob" in dir(glob):
-    glob.iglob = glob.glob
-
-from Options import options
+import time
+import ConfigParser
 
 if 'nt' == os.name:
     import win32file
-    import win32con
 
 if 'posix' == os.name:
     from General import WindowsError
@@ -64,7 +60,7 @@ class OpenFiles:
 
     def scan(self):
         """Update cache"""
-        self.last_scan_time = datetime.datetime.now()
+        self.last_scan_time = time.time()
         self.files = []
         for filename in glob.iglob("/proc/*/fd/*"):
             try:
@@ -78,12 +74,18 @@ class OpenFiles:
 
     def is_open(self, filename):
         """Return boolean whether filename is open by running process"""
-        if None == self.last_scan_time or (datetime.datetime.now() - 
-            self.last_scan_time).seconds > 10:
+        if None == self.last_scan_time or (time.time() -
+            self.last_scan_time) > 10:
             self.scan()
         return filename in self.files
 
 
+def __random_string(length):
+    """Return random alphanumeric characters of given length"""
+    return ''.join(random.choice(string.ascii_letters + '0123456789 _.-') \
+        for i in xrange(length))
+
+
 def bytes_to_human(bytes_i):
     """Display a file size in human terms (megabytes, etc.) using SI standard"""
 
@@ -127,13 +129,77 @@ def children_in_directory(top, list_directories = False):
             yield os.path.join(dirpath, filename)
 
 
+
+def clean_ini(path, section, parameter):
+    """Delete sections and parameters (aka option) in the file"""
+
+    # read file to parser
+    config = ConfigParser.RawConfigParser()
+    fp = codecs.open(path, 'r', encoding='utf_8_sig')
+    config.readfp(fp)
+
+    # change file
+    changed = False
+    if config.has_section(section):
+        if None == parameter:
+            changed = True
+            config.remove_section(section)
+        elif config.has_option(section, parameter):
+            changed = True
+            config.remove_option(section, parameter)
+
+    # write file
+    if changed:
+        from Options import options
+        if options.get('shred'):
+            delete(path, True)
+        fp = codecs.open(path, 'wb', encoding='utf_8')
+        config.write(fp)
+
+
+def clean_json(path, target):
+    """Delete key in the JSON file"""
+    try:
+        import json
+    except:
+        import simplejson as json
+    changed = False
+    targets = target.split('/')
+
+    # read file to parser
+    js = json.load(open(path, 'r'))
+
+    # change file
+    pos = js
+    while True:
+        new_target = targets.pop(0)
+        if not isinstance(pos, dict):
+            break
+        if new_target in pos and len(targets) > 0:
+            # descend
+            pos = pos[new_target]
+        elif new_target in pos:
+            # delete terminal target
+            changed = True
+            del(pos[new_target])
+        else:
+            # target not found
+            break
+        if 0 == len(targets):
+            # target not found
+            break
+
+    if changed:
+        from Options import options
+        if options.get('shred'):
+            delete(path, True)
+        # write file
+        json.dump(js, open(path, 'w'))
+
+
 def delete(path, shred = False):
     """Delete path that is either file, directory, link or FIFO"""
-    try:
-        print u"info: removing '%s'" % path
-    except:
-        p = re.compile('[\x7f-\xff]')
-        print r"info: removing '%s'" % p.sub('?', path)
+    from Options import options
     is_special = False
     if 'posix' == os.name:
         # With certain (relatively rare) files on Windows os.lstat()
@@ -143,8 +209,11 @@ def delete(path, shred = False):
     if is_special:
         os.remove(path)
     elif os.path.isdir(path):
+        delpath = path
+        if shred or options.get('shred'):
+            delpath = wipe_name(path)
         try:
-            os.rmdir(path)
+            os.rmdir(delpath)
         except WindowsError, e:
             # WindowsError: [Error 145] The directory is not empty:
             # 'C:\\Documents and Settings\\username\\Local Settings\\Temp\\NAILogs'
@@ -155,13 +224,19 @@ def delete(path, shred = False):
             else:
                 raise
     elif os.path.isfile(path):
+        # wipe contents
         if shred or options.get('shred'):
             try:
                 wipe_contents(path)
             except IOError, e:
                 # permission denied (13) happens shredding MSIE 8 on Windows 7
                 print "debug: IOError #%s shredding '%s'" % (e.errno, path)
-        os.remove(path)
+        # wipe name
+        if shred or options.get('shred'):
+            os.remove(wipe_name(path))
+        # unlink
+        else:
+            os.remove(path)
     else:
         raise Exception("Unsupported special file type: '%s'" % path)
 
@@ -195,18 +270,14 @@ def exe_exists(pathname):
 
 def execute_sqlite3(path, cmds):
     """Execute 'cmds' on SQLite database 'path'"""
-    try:
-        import sqlite3
-    except ImportError, exc:
-        if sys.version_info[0] == 2 and sys.version_info[1] < 5:
-            raise RuntimeError(_("Cannot import Python module sqlite3: Python 2.5 or later is required."))
-        else:
-            raise exc
+    import sqlite3
     conn = sqlite3.connect(path)
     cursor = conn.cursor()
     for cmd in cmds.split(';'):
         try:
             cursor.execute(cmd)
+        except sqlite3.DatabaseError, exc:
+            raise sqlite3.DatabaseError('%s: %s' % (exc, path))
         except sqlite3.OperationalError, exc:
             if exc.message.find('no such function: ') >= 0:
                 # fixme: determine why randomblob and zeroblob are not available
@@ -227,6 +298,18 @@ def expand_glob_join(pathname1, pathname2):
     return ret
 
 
+def free_space(pathname):
+    """Return free space in bytes"""
+    if 'nt' == os.name:
+        import ctypes
+        free_bytes = ctypes.c_int64()
+        ctypes.windll.kernel32.GetDiskFreeSpaceExW(unicode(pathname), \
+            ctypes.byref(free_bytes), None, None)
+        return free_bytes.value
+    mystat = os.statvfs(pathname)
+    return mystat.f_bfree * mystat.f_bsize
+
+
 def getsize(path):
     """Return the actual file size considering spare files
        and symlinks"""
@@ -265,22 +348,22 @@ def globex(pathname, regex):
                 yield path
 
 
-def human_to_bytes(string, format = 'si'):
+def human_to_bytes(human, hformat = 'si'):
     """Convert a string like 10.2GB into bytes.  By
     default use SI standard (base 10).  The format of the
     GNU command 'du' (base 2) also supported."""
-    if 'si' == format:
+    if 'si' == hformat:
         multiplier = { 'B' : 1, 'kB': 1000, 'MB': 1000**2, \
             'GB': 1000**3, 'TB': 1000**4 }
-        matches = re.findall("^([0-9]*)(\.[0-9]{1,2})?([kMGT]{0,1}B)$", string)
-    elif 'du' == format:
+        matches = re.findall("^([0-9]*)(\.[0-9]{1,2})?([kMGT]{0,1}B)$", human)
+    elif 'du' == hformat:
         multiplier = { 'B' : 1, 'KB': 1024, 'MB': 1024**2, \
             'GB': 1024**3, 'TB': 1024**4 }
-        matches = re.findall("^([0-9]*)(\.[0-9]{1,2})?([KMGT]{0,1}B)$", string)
+        matches = re.findall("^([0-9]*)(\.[0-9]{1,2})?([KMGT]{0,1}B)$", human)
     else:
-        raise ValueError("Invalid format: '%s'" % format)
+        raise ValueError("Invalid format: '%s'" % hformat)
     if [] == matches or 2 > len(matches[0]):
-        raise ValueError("Invalid input for '%s' (format='%s')" % (string, format))
+        raise ValueError("Invalid input for '%s' (hformat='%s')" % (human, hformat))
     return int(float(matches[0][0]+matches[0][1]) * multiplier[matches[0][2]])
 
 
@@ -303,6 +386,7 @@ def listdir(directory):
 
 def whitelisted(path):
     """Check whether this path is whitelisted"""
+    from Options import options
     for pathname in options.get_whitelist_paths():
         if pathname[0] == 'file' and path == pathname[1]:
             return True
@@ -354,15 +438,93 @@ def wipe_contents(path, truncate = True):
     f.close()
 
 
+def wipe_name(pathname1):
+    """Wipe the original filename and return the new pathname"""
+    (head, tail) = os.path.split(pathname1)
+    # reference http://en.wikipedia.org/wiki/Comparison_of_file_systems#Limits
+    maxlen = 226
+    # first, rename to a long name
+    i = 0
+    while True:
+        try:
+            pathname2 = os.path.join(head, __random_string(maxlen))
+            os.rename(pathname1,  pathname2)
+            break
+        except OSError:
+            if maxlen > 10:
+                maxlen -= 10
+            i += 1
+            if i > 100:
+                print 'warning: exhausted long rename: ', pathname1
+                pathname2 = pathname1
+                break
+    # finally, rename to a short name
+    i = 0
+    while True:
+        try:
+            pathname3 = os.path.join(head, __random_string(i + 1))
+            os.rename(pathname2, pathname3)
+            break
+        except:
+            i = i + 1
+            if i > 100:
+                print 'warning: exhausted short rename: ', pathname2
+                pathname3 = pathname2
+                break
+    return pathname3
+
+
 def wipe_path(pathname, idle = False ):
     """Wipe the free space in the path"""
+    def temporaryfile():
+        # reference http://en.wikipedia.org/wiki/Comparison_of_file_systems#Limits
+        maxlen = 245
+        f = None
+        i = 0
+        while True:
+            try:
+                f = tempfile.TemporaryFile(dir = pathname, suffix = __random_string(maxlen))
+                break
+            except OSError, e:
+                if e.errno == 36:
+                    # OSError: [Errno 36] File name too long
+                    if maxlen > 10:
+                        maxlen -= 10
+                    i += 1
+                    if i > 100:
+                        f = tempfile.TemporaryFile(dir = pathname)
+                        break
+                else:
+                    raise
+        return f
+
+
+    def estimate_completion():
+        """Return (percent, seconds) to complete"""
+        remaining_bytes = free_space(pathname)
+        done_bytes = start_free_bytes - remaining_bytes
+        if done_bytes < 0:
+            # mayber user deleted large file after starting wipe
+            done_bytes = 0
+        if 0 == start_free_bytes:
+            done_percent = 0
+        else:
+            done_percent = 1.0 * done_bytes / (start_free_bytes + 1)
+        done_time = time.time() - start_time
+        rate = done_bytes / (done_time + 0.0001) # bytes per second
+        remaining_seconds = int(remaining_bytes / rate)
+        return (done_percent, remaining_seconds)
+
+
     print "debug: wipe_path('%s')" % pathname
     files = []
     total_bytes = 0
+    start_free_bytes = free_space(pathname)
+    start_time = time.time()
     # repeat to clear inodes (Linux) / MFT (Master File Table on Windows)
     while True:
         try:
-            f = tempfile.TemporaryFile(dir = pathname)
+            f = temporaryfile()
         except OSError, e:
             # Linux gives errno 24
             # Windows gives errno 28 No space left on device
@@ -370,15 +532,17 @@ def wipe_path(pathname, idle = False ):
                 break
             else:
                 raise
-        last_idle = datetime.datetime.now()
+        last_idle = time.time()
         # blocks
         blanks = chr(0) * 4096
         try:
             while True:
                 f.write(blanks)
-                if idle and (last_idle - datetime.datetime.now()).seconds > 1:
+                if idle and (time.time() - last_idle) > 2:
                     # Keep the GUI responding, and allow the user to abort.
-                    yield True
+                    # Also display the ETA.
+                    yield estimate_completion()
+                    last_idle = time.time()
         except IOError, e:
             if 28 != e.errno:
                 raise
@@ -399,7 +563,10 @@ def wipe_path(pathname, idle = False ):
         files.append(f)
         total_bytes += f.tell()
     # statistics
-    print 'debug: wrote %d files and %d bytes' % (len(files), total_bytes)
+    elapsed_sec = time.time() - start_time
+    rate_mbs = (total_bytes / (1000*1000) ) / elapsed_sec
+    print 'debug: wrote %d files and %d bytes in %d seconds at %.2f MB/s' % \
+        (len(files), total_bytes, elapsed_sec, rate_mbs)
     # truncate and close files
     for f in files:
         f.truncate(0)
