@@ -390,8 +390,10 @@ class GUI:
 
     def about(self, __event):
         """Create and show the about dialog"""
-        gtk.about_dialog_set_url_hook(lambda dialog, \
-            link: GuiBasic.open_url(link, self.window, False))
+        if 'nt' != os.name and (2, 16, 6) != gtk.gtk_version:
+            # workaround for broken GTK+ (https://bugs.launchpad.net/bleachbit/+bug/797012)
+            gtk.about_dialog_set_url_hook(lambda dialog, \
+                link: GuiBasic.open_url(link, self.window, False))
         dialog = gtk.AboutDialog()
         dialog.set_comments(_("Program to clean unnecessary files"))
         dialog.set_copyright("Copyright (C) 2011 Andrew Ziem")
@@ -499,6 +501,11 @@ class GUI:
         if not paths:
             return
 
+        self.shred_paths(paths)
+
+
+    def shred_paths(self, paths):
+        """Shred file or folders"""
         # create a temporary cleaner object
         backends['_gui'] = Cleaner.create_simple_cleaner(paths)
 
@@ -509,34 +516,25 @@ class GUI:
         if GuiBasic.delete_confirmation_dialog(self.window, mention_preview = False):
             # delete
             self.preview_or_run_operations(True, operations)
-            return
+            return True
+        return False
 
 
     def cb_shred_quit(self, action):
         """Shred settings (for privacy reasons) and quit"""
-        try:
-            FileUtilities.delete(options_file, shred = True)
-            if portable_mode:
-                open(options_file, 'w').write('[Portable]\n')
-            fn = os.path.join(options_dir, "deepscan.sqlite3")
-            if os.path.exists(fn):
-                FileUtilities.delete(fn, shred = True)
-        except:
-            print traceback.print_exc()
-            err = str(sys.exc_info()[1])
-            self.append_text(err, 'error')
+        paths = []
+        if portable_mode:
+            # in portable mode on Windows, the options directory includes executables
+            paths.append(options_file)
+        else:
+            paths.append(options_dir)
+
+        if not self.shred_paths(paths):
+            # aborted
             return
 
-        if not portable_mode:
-            try:
-                os.rmdir(options_dir)
-            except:
-                print traceback.print_exc()
-                err = str(sys.exc_info()[1])
-                self.append_text(err, 'error')
-                while gtk.events_pending():
-                    gtk.main_iteration()
-                time.sleep(3)
+        if portable_mode:
+            open(options_file, 'w').write('[Portable]\n')
 
         gtk.main_quit()
 
@@ -761,7 +759,7 @@ class GUI:
         """Check for software updates in background"""
         import Update
         try:
-            updates = Update.check_updates(options.get('check_beta'))
+            updates = Update.check_updates(options.get('check_beta'), options.get('update_winapp2'), self.append_text)
             if updates:
                 gobject.idle_add(lambda: Update.update_dialog(self.window, updates))
         except:
