@@ -68,7 +68,8 @@ class PreferencesDialog:
     def __toggle_callback(self, cell, path):
         """Callback function to toggle option"""
         options.toggle(path)
-        self.cb_beta.set_sensitive(options.get('check_online_updates'))
+        if online_update_notification_enabled:
+            self.cb_beta.set_sensitive(options.get('check_online_updates'))
         if 'auto_hide' == path:
             self.cb_refresh_operations()
         if 'auto_start' == path:
@@ -91,7 +92,6 @@ class PreferencesDialog:
     def __general_page(self):
         """Return a widget containing the general page"""
 
-
         if 'nt' == os.name:
             swcc = Windows.start_with_computer_check
         if 'posix' == os.name:
@@ -108,11 +108,23 @@ class PreferencesDialog:
             cb_updates.set_tooltip_text(_("If an update is found, you will be given the option to view information about it.  Then, you may manually download and install the update."))
             vbox.pack_start(cb_updates, False)
 
+            updates_box = gtk.VBox()
+            updates_box.set_border_width(10)
+
             self.cb_beta = gtk.CheckButton(_("Check for new beta releases"))
             self.cb_beta.set_active(options.get('check_beta'))
             self.cb_beta.set_sensitive(options.get('check_online_updates'))
             self.cb_beta.connect('toggled', self.__toggle_callback, 'check_beta')
-            vbox.pack_start(self.cb_beta, False)
+            updates_box.pack_start(self.cb_beta, False)
+
+            if 'nt' == os.name:
+                self.cb_winapp2 = gtk.CheckButton(_("Download and update cleaners from community (winapp2.ini)"))
+                self.cb_winapp2.set_active(options.get('update_winapp2'))
+                self.cb_winapp2.set_sensitive(options.get('check_online_updates'))
+                self.cb_winapp2.connect('toggled', self.__toggle_callback, 'update_winapp2')
+                updates_box.pack_start(self.cb_winapp2, False)
+
+            vbox.pack_start(updates_box, False)
 
         # TRANSLATORS: This means to hide cleaners which would do
         # nothing.  For example, if Firefox were never used on
