@@ -31,7 +31,9 @@ import Common
 import General
 
 
-boolean_keys = ('auto_hide', 'auto_start', 'check_beta', 'check_online_updates', 'first_start', 'shred')
+boolean_keys = ['auto_hide', 'auto_start', 'check_beta', 'check_online_updates', 'first_start', 'shred']
+if 'nt' == os.name:
+    boolean_keys.append('update_winapp2')
 
 
 
@@ -70,6 +72,8 @@ class Options:
 
     def get(self, option, section = 'bleachbit'):
         """Retrieve a general option"""
+        if not 'nt' == os.name and 'update_winapp2' == option:
+            return False
         if section == 'hashpath' and option[1] == ':':
             option = option[0] + option[2:]
         if option in boolean_keys:
@@ -154,6 +158,8 @@ class Options:
         self.__set_default("check_beta", False)
         self.__set_default("check_online_updates", True)
         self.__set_default("shred", False)
+        if 'nt' == os.name:
+            self.__set_default("update_winapp2", False)
 
         if not self.config.has_section('preserve_languages'):
             lang = Common.user_locale
