@@ -25,6 +25,9 @@ Check for updates via the Internet
 
 
 import gtk
+import hashlib
+import os
+import os.path
 import platform
 import socket
 import sys
@@ -37,6 +40,41 @@ from Common import _
 from GuiBasic import open_url
 
 
+def update_winapp2(url, hash_expected, append_text):
+    """Download latest winapp2.ini file.  Hash is sha512 or None to disable checks"""
+    # first, determine whether an update is necessary
+    from Common import personal_cleaners_dir
+    fn = os.path.join(personal_cleaners_dir, 'winapp2.ini')
+    delete_current = False
+    if os.path.exists(fn):
+        f = open(fn, 'r')
+        hash_current = hashlib.sha512(f.read()).hexdigest()
+        if not hash_expected or hash_current == hash_expected:
+            # update is same as current
+            return
+        f.close()
+        delete_current = True
+    # download update
+    opener = urllib2.build_opener()
+    opener.addheaders =[('User-Agent', user_agent())]
+    doc = opener.open(url).read()
+    # verify hash
+    hash_actual = hashlib.sha512(doc).hexdigest()
+    if hash_expected and not hash_actual == hash_expected:
+        raise RuntimeError("hash for %s actually %s instead of %s" % \
+            (url, hash_actual, hash_expected))
+    # delete current
+    if delete_current:
+        from FileUtilities import delete
+        delete(fn, True)
+    # write file
+    if not os.path.exists(personal_cleaners_dir):
+        os.mkdir(personal_cleaners_dir)
+    f = open(fn, 'w')
+    f.write(doc)
+    append_text(_('New winapp2.ini was downloaded.'))
+
+
 def user_agent():
     """Return the user agent string"""
     __platform = platform.system() # Linux or Windows
@@ -99,11 +137,11 @@ def update_dialog(parent, updates):
 
 
 
-def check_updates(check_beta):
+def check_updates(check_beta, check_winapp2, append_text):
     """Check for updates via the Internet"""
     opener = urllib2.build_opener()
-    opener.addheaders = [('User-Agent', user_agent())]
     socket.setdefaulttimeout(Common.socket_timeout)
+    opener.addheaders = [('User-Agent', user_agent())]
     handle = opener.open(Common.update_check_url)
     doc = handle.read()
     try:
@@ -121,6 +159,13 @@ def check_updates(check_beta):
     stable = parse_updates(dom.getElementsByTagName("stable"))
     beta = parse_updates(dom.getElementsByTagName("beta"))
 
+    wa_element = dom.getElementsByTagName('winapp2')
+    if check_winapp2 and wa_element:
+        wa_sha512 = wa_element[0].getAttribute('sha512')
+        wa_url = wa_element[0].getAttribute('url')
+        update_winapp2(wa_url, wa_sha512, append_text)
+
+
     dom.unlink()
 
     if stable and beta and check_beta:
