@@ -195,11 +195,16 @@ def delete_updates():
     if not dirs:
         # if nothing to delete, then also do not restart service
         return
+
+    import win32serviceutil
+    wu_running = win32serviceutil.QueryServiceStatus('wuauserv')[1] == 4
+
     args = ['net', 'stop', 'wuauserv']
     def wu_service():
         General.run_external(args)
         return 0
-    yield Command.Function(None, wu_service, " ".join(args))
+    if wu_running:
+        yield Command.Function(None, wu_service, " ".join(args))
 
     for path1 in dirs:
         for path2 in FileUtilities.children_in_directory(path1, True):
@@ -208,7 +213,8 @@ def delete_updates():
             yield Command.Delete(path1)
 
     args = ['net', 'start', 'wuauserv']
-    yield Command.Function(None, wu_service, " ".join(args))
+    if wu_running:
+        yield Command.Function(None, wu_service, " ".join(args))
 
 
 def detect_registry_key(parent_key):
@@ -281,6 +287,15 @@ def elevate_privileges():
     return False
 
 
+def empty_recycle_bin(drive, really_delete):
+    """Empty the recycle bin or preview its size"""
+    bytes_used = shell.SHQueryRecycleBin(drive)[0]
+    if really_delete and bytes_used > 0:
+        # Trying to delete an empty Recycle Bin on Vista/7 causes a 'catastrophic failure'
+        flags = shellcon.SHERB_NOSOUND | shellcon.SHERB_NOCONFIRMATION | shellcon.SHERB_NOPROGRESSUI
+        shell.SHEmptyRecycleBin(None, drive, flags)
+    return bytes_used
+
 
 def enumerate_processes():
     """Return list of module names (e.g., firefox.exe) of running
@@ -298,7 +313,6 @@ def enumerate_processes():
     return r
 
 
-
 def enumerate_processes_win32():
     """Return list of module names (e.g., firefox.exe) of running
     processes
@@ -342,8 +356,6 @@ def enumerate_processes_win32():
     return modnames
 
 
-
-
 def enumerate_processes_wmic():
     """Return list of module names (e.g., firefox.exe) of running
     processes
@@ -361,6 +373,8 @@ def enumerate_processes_wmic():
     return sorted(modnames)
 
 
+
+
 def get_fixed_drives():
     """Yield each fixed drive"""
     for drive in win32api.GetLogicalDriveStrings().split('\x00'):
@@ -368,14 +382,12 @@ def get_fixed_drives():
             yield drive
 
 
-def empty_recycle_bin(drive, really_delete):
-    """Empty the recycle bin or preview its size"""
-    bytes_used = shell.SHQueryRecycleBin(drive)[0]
-    if really_delete and bytes_used > 0:
-        # Trying to delete an empty Recycle Bin on Vista/7 causes a 'catastrophic failure'
-        flags = shellcon.SHERB_NOSOUND | shellcon.SHERB_NOCONFIRMATION | shellcon.SHERB_NOPROGRESSUI
-        shell.SHEmptyRecycleBin(None, drive, flags)
-    return bytes_used
+def path_on_network(path):
+    """Check whether 'path' is on a network drive"""
+    if len(os.path.splitunc(path)[0]) > 0:
+        return True
+    drive = os.path.splitdrive(path)[0] + '\\'
+    return win32file.GetDriveType(drive) == win32file.DRIVE_REMOTE
 
 
 def split_registry_key(full_key):
@@ -413,9 +425,4 @@ def start_with_computer_check():
     return os.path.lexists(Common.autostart_path)
 
 
-def path_on_network(path):
-    """Check whether 'path' is on a network drive"""
-    if len(os.path.splitunc(path)[0]) > 0:
-        return True
-    drive = os.path.splitdrive(path)[0] + '\\'
-    return win32file.GetDriveType(drive) == win32file.DRIVE_REMOTE
+
