@@ -1,7 +1,7 @@
 # vim: ts=4:sw=4:expandtab
 
 ## BleachBit
-## Copyright (C) 2010 Andrew Ziem
+## Copyright (C) 2011 Andrew Ziem
 ## http://bleachbit.sourceforge.net
 ##
 ## This program is free software: you can redistribute it and/or modify
@@ -31,7 +31,7 @@ import traceback
 import DeepScan
 import FileUtilities
 from Cleaner import backends
-from Common import _
+from Common import _, ungettext
 
 if 'nt' == os.name:
     import Windows
@@ -86,9 +86,11 @@ class Worker:
         ret = None
         try:
             for ret in cmd.execute(self.really_delete):
-                if True == ret:
-                    # temporarily pass control to the GTK idle loop
-                    yield True
+                if True == ret or isinstance(ret, tuple):
+                    # Temporarily pass control to the GTK idle loop,
+                    # allow user to abort, and
+                    # display progress (if applicable).
+                    yield ret
         except SystemExit:
             pass
         except Exception, e:
@@ -164,51 +166,66 @@ class Worker:
 
 
     def run(self):
-        """Perform the main cleaning process"""
-        count = 0
+        """Perform the main cleaning process which has these phases
+        1. General cleaning
+        2. Deep scan
+        3. Memory
+        4. Free disk space"""
         self.deepscans = {}
+        # prioritize
+        self.delayed_ops = []
         for operation in self.operations:
-            self.ui.update_progress_bar(1.0 * count / len(self.operations))
-            name = backends[operation].get_name()
-            if self.really_delete:
-                # TRANSLATORS: %s is replaced with Firefox, System, etc.
-                msg = _("Please wait.  Cleaning %s.") % name
-            else:
-                # TRANSLATORS: %s is replaced with Firefox, System, etc.
-                msg = _("Please wait.  Previewing %s.") % name
-            self.ui.update_progress_bar(msg)
-            yield True # show the progress bar message now
-            try:
-                for dummy in self.clean_operation(operation):
-                    yield True
-            except:
-                self.print_exception(operation)
+            delayables = ['free_disk_space', 'memory']
+            for delayable in delayables:
+                if operation not in ('system', '_gui'):
+                    continue
+                if delayable in self.operations[operation]:
+                    i = self.operations[operation].index(delayable)
+                    del self.operations[operation][i]
+                    priority = 99
+                    if 'free_disk_space' == delayable:
+                        priority = 100
+                    new_op = (priority, { operation : [ delayable ] } )
+                    self.delayed_ops.append( new_op )
 
-            count += 1
+        # standard operations
+        for dummy in self.run_operations(self.operations):
+            # yield to GTK+ idle loop
+            yield True
 
         # run deep scan
-        print 'debug: deepscans=', self.deepscans
-        # TRANSLATORS: The "deep scan" feature searches over broad
-        # areas of the file system such as the user's whole home directory
-        # or all the system executables.
-        self.ui.update_progress_bar("Please wait.  Running deep scan.")
-        yield True # allow GTK to update the screen
-        ds = DeepScan.DeepScan()
-        for (path, dsdict) in self.deepscans.iteritems():
-            print 'debug: deepscan path=',path
-            print 'debug: deepscan dict=', dsdict
-            for dsdict2 in dsdict:
-                ds.add_search(path, dsdict2['regex'])
+        if self.deepscans:
+            for dummy in self.run_deep_scan():
+                yield dummy
+
+        # delayed operations
+        for op in sorted(self.delayed_ops):
+            operation = op[1].keys()[0]
+            for option_id in op[1].values()[0]:
+                self.ui.update_progress_bar(0.0)
+                if 'free_disk_space' == option_id:
+                    # TRANSLATORS: 'free' means 'unallocated'
+                    msg = _("Please wait.  Wiping free disk space.")
+                elif 'memory' == option_id:
+                    msg = _("Please wait.  Cleaning %s.") % _("Memory")
+                else:
+                    raise RuntimeError("Unexpected option_id in delayed ops")
+                self.ui.update_progress_bar(msg)
+                for cmd in backends[operation].get_commands(option_id):
+                    for ret in self.execute(cmd):
+                        if isinstance(ret, tuple):
+                            # Display progress (for free disk space)
+                            self.ui.update_progress_bar(ret[0])
+                            mins = int(ret[1] / 60)
+                            msg2 = ungettext("About %d minute remaining.", \
+                                "About %d minutes remaining.", mins) \
+                                % mins
+                            self.ui.update_progress_bar(msg + ' ' + msg2)
+                        if True == ret or isinstance(ret, tuple):
+                            # Return control to PyGTK idle loop to keep
+                            # it responding and allow the user to abort.
+                            yield True
 
-        for path in ds.scan():
-            if True == path:
-                yield True
-                continue
-            # fixme: support non-delete commands
-            import Command
-            cmd = Command.Delete(path)
-            for ret in self.execute(cmd):
-                yield True
 
         # print final stats
         bytes_delete = FileUtilities.bytes_to_human(self.total_bytes)
@@ -244,4 +261,49 @@ class Worker:
         yield False
 
 
+    def run_deep_scan(self):
+        """Run deep scans"""
+        print 'debug: deepscans=', self.deepscans
+        # TRANSLATORS: The "deep scan" feature searches over broad
+        # areas of the file system such as the user's whole home directory
+        # or all the system executables.
+        self.ui.update_progress_bar(_("Please wait.  Running deep scan."))
+        yield True # allow GTK to update the screen
+        ds = DeepScan.DeepScan()
+        for (path, dsdict) in self.deepscans.iteritems():
+            print 'debug: deepscan path=', path
+            print 'debug: deepscan dict=', dsdict
+            for dsdict2 in dsdict:
+                ds.add_search(path, dsdict2['regex'])
 
+        for path in ds.scan():
+            if True == path:
+                yield True
+                continue
+            # fixme: support non-delete commands
+            import Command
+            cmd = Command.Delete(path)
+            for ret in self.execute(cmd):
+                yield True
+
+    def run_operations(self, my_operations):
+        """Run a set of operations (general, memory, free disk space)"""
+        count = 0
+        for operation in my_operations:
+            self.ui.update_progress_bar(1.0 * count / len(my_operations))
+            name = backends[operation].get_name()
+            if self.really_delete:
+                # TRANSLATORS: %s is replaced with Firefox, System, etc.
+                msg = _("Please wait.  Cleaning %s.") % name
+            else:
+                # TRANSLATORS: %s is replaced with Firefox, System, etc.
+                msg = _("Please wait.  Previewing %s.") % name
+            self.ui.update_progress_bar(msg)
+            yield True # show the progress bar message now
+            try:
+                for dummy in self.clean_operation(operation):
+                    yield True
+            except:
+                self.print_exception(operation)
+
+            count += 1
