@@ -73,7 +73,7 @@ class CLITestCase(unittest.TestCase):
         tests = ( ( ['adobe_reader.*'], {'adobe_reader' : [ u'cache', u'mru', u'tmp' ] } ),
             ( ['adobe_reader.mru'], {'adobe_reader' : [ u'mru' ] } ) )
         for test in tests:
-            o = args_to_operations(test[0])
+            o = args_to_operations(test[0], False)
             self.assertEqual(o, test[1])
 
 
@@ -150,7 +150,7 @@ class CLITestCase(unittest.TestCase):
         FileUtilities.delete = dummy_delete
         FileUtilities.delete(filename)
         self.assert_(os.path.exists(filename))
-        operations = args_to_operations(['system.tmp'])
+        operations = args_to_operations(['system.tmp'], False)
         preview_or_delete(operations, True)
         FileUtilities.delete = save_delete
         self.assert_(filename in deleted_paths, \
